import mx.central.AgentManager;
/**
 * A Central agent that queries stock data and manages shared object storgae
 * for the StockWatcher application.
 */
class StockAgent implements mx.central.Agent {

	/**
	 * A reference to the AgentManager used for accessing Notices
	 */
    var oManager:AgentManager; 
    
    /**
	 *  Sets the think interval to every 15 minutes
	 */
    var kThinkFrequency:Number = 15 * 60 * 1000;
	
	/**
	 *  The think interval object
	 */
    var oThinkInterval = null;

	/**
	 * Reference to the service object that does most of the work
	 */
    var oStockService:StockService; 

	/**
	 * Default constructor
	 */
	function StockAgent()
    {

    }

    /**
	 * Central initialization
	 */
    function onActivate(agentManager:AgentManager, agentID:Number,
                        initialData:Object):Void
    {
		StockUtils.trace("onActivate()", "StockAgent");
        // keep a reference to the Agent Manager to use its API
        oManager = agentManager;

        // now create the LCService service object
        oStockService = new StockService(this);
		
        onNetworkChange(oManager.isConnected());
    }
	
	/**
	 * Called just before the agent instance is unloaded.
	 **/	
    function onDeactivate(Void):Void
    {
        stopThinking();
    }

	/**
	 * Called when the connection status changes to online or offline
	 **/
	function onNetworkChange(connected:Boolean):Void
    {
        oStockService.bOnline = connected;
	
        //  do periodic checking of data only when user says we're online
        if (connected) {
            startThinking();
        }
        else {
            stopThinking();
        }
    }

	/**
	 * Called when a notice is dismissed.
	 */
    function onNoticeEvent(event:Object, noticeData:Object,
                           initialData:Object):Void
    {
        oStockService.processNoticeEvent(noticeData);
        
    }
	/**
	 * Called when the application is being uninstalled from Central.
	 **/
    function onUninstall(Void):Void {
		// not implemented
	}
    
    /* ------------------------------------------------------------------
     * Main agent activity
	 * ------------------------------------------------------------------*/

	/**
	 * Does a stock update, then turns on the timer
	 */
    function startThinking()
    {
		StockUtils.trace("startThinking()", "StockAgent");
        var sa:StockAgent = this;
	
        oStockService.think();

        // don't keep setting timers if one is already set
        if (oThinkInterval) return;
        
        // think again later
        //var thinkThunk = function() { 
			//sa.gStockService.think(); 
		//}
        oThinkInterval = setInterval(oStockService.think, kThinkFrequency);
    }

    /**
	 * Disables the timer
	 */
    function stopThinking()
    {
        if (oThinkInterval) {
            clearInterval(oThinkInterval);
            oThinkInterval = null;
        }
    }

    /**
	 * Posts a nicely formatted notice with stock data
	 */
    function postNotice(symbol:String, name:String,
                        description:String, alert:Boolean):Number
    {
        var notice:Object = new Object();
        notice.name = name;
        notice.description = description;
        
        notice.alert = alert;
        notice.engageString = "Show";
        notice.navigate = true;
        
        var noticeID:Number =
            oManager.addNotice(notice, {symbol: symbol});
        return noticeID;
    }

    /**
	 * Removes the given notice
	 */
    function removeNotice(noticeID:Number) {
        oManager.removeNotice(noticeID);
    }


}
