
//
// Copyright 2003, 2004 Macromedia

import mx.central.Central;
import mx.central.Console;

import mx.central.data.LCService;
import mx.central.data.LCDataProvider;
import mx.central.data.SelectedItem;

import mx.utils.Delegate;

import mx.core.UIObject;

/**
 * The StockWatcher Pod class.
 * Extends UIObject so that it can support V2 Window pop-ups (for future use).
 */
class StockPod extends UIObject implements mx.central.Pod {
	/**
	 * Instance variables
     */
	var sAppName:String = "StockPod";
	
	/**
	 * Reference to the Console
	 */
	var oConsole:Console;
	/**
	 * Local persistent storage for last stock symbol searched for, etc.
	 */
	var oStorage;
	/**
	 * LCService client object to communicate with app and agent
	 */
	var oConn;
	/**
	 * Whether the pod is currently connected to the network
	 */		
	var bIsOnline:Boolean = true;
	/**
	 * Reference to the main movie clip for the pod
	 */	
	var oRoot:MovieClip;

	var fGoButton:mx.controls.Button;
	var fSymbol:mx.controls.TextInput;
	var fTossHitArea:MovieClip;
	var fTossButton:mx.central.controls.MIconButton;
	
	var fLastTrade:TextField;
	var fTradeTime:TextField;
	var fChange:TextField;
	var fChangePercent:TextField;
	var fName:TextField;
	var fArrow:MovieClip;
	
	var bIsInitialized:Boolean = false;
	
	/**
	 * Constructor 
	 */
    function StockPod(mc:MovieClip) {
        oRoot = mc;
    }
	
	/**
	 * Initialization method is called automatically since this extends UIObject.
	 */
	function init() {
		StockUtils.trace("init()","StockPod");
	}
	
	/**
	 * Called automatically (since this extends UIObject) when the pod's display is ready to be drawn
	 */	
	function draw() {
		StockUtils.trace("draw()","StockPod");
		
		if (!bIsInitialized) {
			// stylize the pod's components
			StockStyles.defineStyles();
			fGoButton.setStyle("styleName", StockStyles.kStyleName);
			fSymbol.setStyle("styleName", StockStyles.kStyleName);
			
			var tf:TextFormat = StockStyles.getTextFormat();
			fName.setTextFormat(tf);
			fName.setNewTextFormat(tf);
			fLastTrade.setTextFormat(tf);
			fLastTrade.setNewTextFormat(tf);
			fTradeTime.setTextFormat(tf);
			fTradeTime.setNewTextFormat(tf);
			fChange.setTextFormat(tf);
			fChange.setNewTextFormat(tf);	
			fChangePercent.setTextFormat(tf);
			fChangePercent.setNewTextFormat(tf);	
			
			// start the ball rolling
			Central.initPod(this, this);
			
			bIsInitialized = true;
		}
	}
	
	function initStorage()
	{
		oStorage = SharedObject.getLocal(sAppName);
		if (oStorage.data.searchFor == undefined)
		{
			oStorage.data.searchFor = "";
		}
	}
	
	//-------------------------------------------------------------------
	// Central Event Handling
	//
	
	function onActivate(console:Console, podID:Number, viewerID:Number, position:Number, baseTabIndex:Number, initialData:Object):Void
	{
		StockUtils.trace("onActivate()","StockPod");
		oConsole = console;
		sAppName = sAppName + "_" + viewerID;
		
		// hide the up/down stock arrow until we have data to show
		fArrow._visible = false; 
		
		initStorage();
		
		// start agent in case it's not running
        oConsole.startAgent();
		
		// sets up event handlers
		fGoButton.addEventListener("click", Delegate.create(this, invokeSearch));
		fSymbol.addEventListener("enter", Delegate.create(this, invokeSearch));
		
		// clicking the toss hit area launches the app
		fTossHitArea.onRelease = Delegate.create(this,loadApp);
		fTossButton.onRelease = Delegate.create(this,loadApp);
		
		// establish local connection to agent for data
		oConn = LCService.createClient( StockInterfaces, sAppName, this, false );
		oConn.allowDomain = function(domain:String):Boolean {
			return true;
		};	
	
		onNetworkChange(oConsole.isConnected());
		
		// restore last search term
		if (oStorage.data.searchFor != "") {
			fSymbol.text = oStorage.data.searchFor;
			if (oStorage.data.lastResult != undefined) {
				setTicker(oStorage.data.lastResult);
			} else if (bIsOnline) {
				doSearch();
			}
		}
	}
	
	/**
	 * Called if the LCService connection fails
	 */
	public function createClient_Status (clientObject:Object) {
		StockUtils.trace("LCService.createClient failed.");
	}
	
	/**
	 * Called when the LCService connection to the AIM service is successful
	 */
	function createClient_Result (proxyObject:Object) {
		StockUtils.trace("LCService.createClient succeeded.");
	}
	
	function onDeactivate(Void):Void
	{
		oConn.close();
	}
	
	function onNetworkChange(bOnline:Boolean):Void
	{
		bIsOnline = bOnline;
		
		// do request to refresh current information
		doSearch();
	}
	
	//-------------------------------------------------------------------
	// Button Events
	//
	
	/**
	 * Handles clicks on the go button
	 */
	function invokeSearch(uiEvent:Object)
	{
		doSearch();
	}
	
	function doSearch()
	{
		StockUtils.trace("doSearch()", "StockPod");
		var symbol = fSymbol.text;
		
		if (symbol != undefined && symbol != "") {
			// DEBUG: set value of a stock for testing
			if (symbol.substr(0,4) == "set ")
			{
				var symbol = fSymbol.text.substr(4,4);
				var value = fSymbol.text.substr(9);
				oConn.debugSetStock({symbol: symbol, value: value});
				fName.text = (symbol == "free" ? "test values freed" : symbol+" set to");
				fLastTrade.text = value;
				return;
			}
			
			// remember this search term for next launch
			oStorage.data.searchFor = symbol;
			oConn.stockSearch(symbol);
		}
	}

	// should be called automatically
	function stockSearch_Result(result:StockItem)
	{
		StockUtils.trace("stockSearch_Result()", "StockPod");
		//StockUtils.traceProps("result=", result);
		setTicker(result);
	}
	
	// click anywhere on the toss icon to launch the stock app
	public function loadApp()
	{
		StockUtils.trace("loadApp()", "StockPod");
		oConsole.loadApplication();
	}
	
	//-------------------------------------------------------------------
	// View Updating
	//
	
	function setTicker(result:StockItem):Void
	{
		StockUtils.trace("setTicker(" + result.symbol + "), fSymbol=" + fSymbol.text.toUpperCase(), "StockPod");
		//StockUtils.traceProps("result=", result);
		
		if (result != undefined) {
			// if this result matches our current search, update the display 
			// (it might be a result from another pod's search)
			if (result.symbol.toUpperCase() == fSymbol.text.toUpperCase() ||
					this.fSymbol.text == "")
			{
				oStorage.data.lastResult = result;
				with (result)
				{
					this.fName.text = company;
					
					// if we have data, show it, otherwise empty fields
					if (lastPrice != undefined)
					{
						this.fLastTrade.text = lastPrice;
						this.fTradeTime.text = tradedTime+" ET";
						this.fChange.text = change;
						this.fChangePercent.text = changePercent;
						this.fArrow.gotoAndStop((change > 0 ? "up" : "down"));
						this.fArrow._visible = true;
					}
					else
					{
						this.fLastTrade.text = "";
						this.fTradeTime.text = "";
						this.fChange.text = "";
						this.fChangePercent.text = "";
						this.fArrow._visible = false;
					}
				}
			}
		}
	}
	
	// when the model changes, the agent will call this to get pods to reload their data
	function refreshPods():Void
	{
		//tracer.log("refreshPod search="+oStorage.data.searchFor);
		oConn.getCompany(oStorage.data.searchFor);
	}
	
	function getCompany_Result(result:StockItem):Void
	{
		//tracer.log("getCompany_Result symbol="+result.symbol+" value="+value);
		StockUtils.trace("getCompany_Result()", "StockPod");
		setTicker(result);
	}
	
	function setCompany(stock:StockItem):Void
	{
		StockUtils.trace("setCompany()", "StockPod");
		
		// update ticker with info
		setTicker(stock);
	}
	
	/** 
	 * Other methods from the mx.central.Pod interface
	 */
	function getLastTabIndex(Void):Number {
		// not implemented
		return 1;
	}
	
	function onNoticeEvent (event:Object, noticeData:Object, initialData:Object):Void {
		// not implemented
	}
	
	function onPositionChange (newPosition:Number):Void {
		// not implemented
	}
	
	function setBaseTabIndex (newIndex:Number):Void {
		// not implemented
	}
	
	function onSelectedItem (data:Array):Void {
		// not implemented
	}
	
	

}
